@@ -11,11 +11,11 @@ from typing import (
     runtime_checkable,
 )
 
-from fastapi import APIRouter, HTTPException, Request, Response
+from fastapi import APIRouter, BackgroundTasks, HTTPException, Request, Response
 from fastapi.routing import APIRoute
 from pydantic import BaseModel
 
-from fastapi_resources.resources.sqlmodel.exceptions import NotFound
+from fastapi_resources.resources.sqlalchemy.exceptions import NotFound
 from fastapi_resources.resources.types import ResourceProtocol
 from fastapi_resources.routers import decorators
 
@@ -246,7 +246,13 @@ class ResourceRouter(APIRouter, Generic[TResource]):
 
         return update, relationships
 
-    def perform_create(
+    async def _await_if_necessary(self, result):
+        if inspect.isawaitable(result):
+            return await result
+
+        return result
+
+    async def perform_create(
         self,
         request: Request,
         resource: TResource,
@@ -256,9 +262,11 @@ class ResourceRouter(APIRouter, Generic[TResource]):
         if not resource.create:
             raise NotImplementedError("Resource.create not implemented")
 
-        return resource.create(attributes=attributes, relationships=relationships)
+        return await self._await_if_necessary(
+            resource.create(attributes=attributes, relationships=relationships)
+        )
 
-    def perform_update(
+    async def perform_update(
         self,
         request: Request,
         resource: TResource,
@@ -269,15 +277,17 @@ class ResourceRouter(APIRouter, Generic[TResource]):
         if not resource.update:
             raise NotImplementedError("Resource.update not implemented")
 
-        return resource.update(
-            id=id, attributes=attributes, relationships=relationships
+        return await self._await_if_necessary(
+            resource.update(id=id, attributes=attributes, relationships=relationships)
         )
 
-    def perform_delete(self, request: Request, resource: TResource, id: str | int):
+    async def perform_delete(
+        self, request: Request, resource: TResource, id: str | int
+    ):
         if not resource.delete:
             raise NotImplementedError("Resource.delete not implemented")
 
-        return resource.delete(id=id)
+        return await self._await_if_necessary(resource.delete(id=id))
 
     async def _retrieve(self, *, id: Union[int, str], request: Request):
         resource = self.get_resource(request=request)
@@ -285,7 +295,8 @@ class ResourceRouter(APIRouter, Generic[TResource]):
             raise NotImplementedError("Resource.retrieve not implemented")
 
         row = resource.retrieve(id=id)
-        return self.build_response(rows=row, resource=resource, request=request)
+        res = self.build_response(rows=row, resource=resource, request=request)
+        return res
 
     async def _list(self, *, request: Request):
         resource = self.get_resource(request=request)
@@ -299,43 +310,87 @@ class ResourceRouter(APIRouter, Generic[TResource]):
             rows=rows, resource=resource, request=request, count=count, next=next
         )
 
-    async def _create(self, *, create: TCreatePayload, request: Request):
-        resource = self.get_resource(request=request)
+    def _process_tasks(self, background_tasks: BackgroundTasks, resource: TResource):
+        for task in resource.tasks:
+            background_tasks.add_task(
+                task.func,
+                *task.args,
+                **task.keywords,
+            )
 
-        attributes, relationships = self.parse_update(
-            resource=resource, update=create.dict(exclude_unset=True)
-        )
-
-        row = self.perform_create(
-            request=request,
-            resource=resource,
-            attributes=attributes,
-            relationships=relationships,
-        )
-
-        return self.build_response(rows=row, resource=resource, request=request)
-
-    async def _update(
-        self, *, id: Union[int, str], update: TUpdatePayload, request: Request
+    async def _create(
+        self,
+        *,
+        create: TCreatePayload,
+        request: Request,
+        background_tasks: BackgroundTasks,
     ):
         resource = self.get_resource(request=request)
 
         attributes, relationships = self.parse_update(
-            resource=resource, update=update.dict(exclude_unset=True)
-        )
-        row = self.perform_update(
-            request=request,
-            resource=resource,
-            attributes=attributes,
-            relationships=relationships,
-            id=id,
+            resource=resource, update=create.model_dump(exclude_unset=True)
         )
 
+        try:
+            row = await self.perform_create(
+                request=request,
+                resource=resource,
+                attributes=attributes,
+                relationships=relationships,
+            )
+        finally:
+            self._process_tasks(
+                background_tasks=background_tasks,
+                resource=resource,
+            )
+
         return self.build_response(rows=row, resource=resource, request=request)
 
-    async def _delete(self, *, id: Union[int, str], request: Request):
+    async def _update(
+        self,
+        *,
+        id: Union[int, str],
+        update: TUpdatePayload,
+        request: Request,
+        background_tasks: BackgroundTasks,
+    ):
         resource = self.get_resource(request=request)
 
-        self.perform_delete(request=request, resource=resource, id=id)
+        attributes, relationships = self.parse_update(
+            resource=resource, update=update.model_dump(exclude_unset=True)
+        )
+
+        try:
+            row = await self.perform_update(
+                request=request,
+                resource=resource,
+                attributes=attributes,
+                relationships=relationships,
+                id=id,
+            )
+        finally:
+            self._process_tasks(
+                background_tasks=background_tasks,
+                resource=resource,
+            )
+
+        return self.build_response(rows=row, resource=resource, request=request)
+
+    async def _delete(
+        self,
+        *,
+        id: Union[int, str],
+        request: Request,
+        background_tasks: BackgroundTasks,
+    ):
+        resource = self.get_resource(request=request)
+
+        try:
+            await self.perform_delete(request=request, resource=resource, id=id)
+        finally:
+            self._process_tasks(
+                background_tasks=background_tasks,
+                resource=resource,
+            )
 
         return Response(status_code=204)
